#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2021,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##
from typing import List

from aceql import Connection
from aceql._private.table_names_dto import TableNamesDto
from aceql._private.jdbc_database_meta_data_dto import JdbcDatabaseMetaDataDto
from aceql._private.table_dto import TableDto
from aceql.metadata.table import Table
from aceql.metadata.jdbc_database_meta_data import JdbcDatabaseMetaData


class RemoteDatabaseMetaData(object):

    """Allows to retrieve metadata info from the remote SQL database."""

    def __init__(self, connection: Connection):
        self.__connection = connection
        self.__aceql_http_api = self.__connection._get_aceql_http_api

    def db_schema_download(self, file: str, file_format: str = None, table_name: str = None):
        """
        Downloads the schema extract for a table name in the specified HTML or Text format.

        Parameters
        ----------
        file : str
            The filename to store the output schema in
        file_format
            the format of the output "html" or "text". Defaults to "html"
        table_name
            if specified, output will be done only for the table
        """

        if file is None:
            raise TypeError("file is null!")

        if file_format is None:
            file_format = "html"

        if file_format != "html" and file_format != "text":
            raise TypeError("Invalid format value. Must be \"file\" or \"text\". Is: " + file_format)

        response = self.__aceql_http_api.db_schema_download(file_format, table_name)

        with open(file, 'wb') as fd:
            for chunk in response.iter_content(chunk_size=2048):
                fd.write(chunk)

    def get_jdbc_database_meta_data(self) -> JdbcDatabaseMetaData:
        """
        Returns the basic meta data values of the remote database, as sent by the the remote JDBC Driver of the remote database.
        :return:
        the basic meta data values sent by the the remote JDBC Driver of the remote database.
        """
        jdbc_database_meta_data_holder : JdbcDatabaseMetaDataDto = self.__aceql_http_api.get_db_metadata()
        return jdbc_database_meta_data_holder.jdbcDatabaseMetaData

    def get_table_names(self, table_type: str =None) -> List[str]:
        """
        Returns The list of tables
        :param table_type:
        the table type. Possible values: "table","view", etc. Defaults to all types if null.

        :return:
        The list of tables
        """
        table_names_dto : TableNamesDto = self.__aceql_http_api.get_table_names(table_type)
        table_names: List[str] = table_names_dto.tableNames
        return table_names

    def get_table(self, name: str) -> Table:
        """
        Returns the Table details
        :param name:
        The name table to get without any prefix/dot

        :return:
        The Table details
        """
        if name is None:
            raise TypeError("table name is null!")

        table_dto: TableDto = self.__aceql_http_api.get_table(name)
        table: Table = table_dto.table
        return table



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2021,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##

"""aceql library. Allows to wrap the AceQL HTTP APIs to access remote SQL databases.

    The library supports Python 3.6 to 3.9.
    It provides a SQL interface compliant with the DB-API 2.0 specification described by PEP 249.
"""

from aceql.connection import Connection
from aceql.progress_indicator import ProgressIndicator
from aceql.sql_null_type import SqlNullType
from aceql.cursor import Cursor
from aceql.error import Error
from aceql.proxy_auth import ProxyAuth
from aceql.connection_options import ConnectionOptions

from aceql.metadata.exportedkey import ExportedKey
from aceql.metadata.foreignkey import ForeignKey
from aceql.metadata.importedkey import ImportedKey
from aceql.metadata.index import Index
from aceql.metadata.jdbc_database_meta_data import JdbcDatabaseMetaData
from aceql.metadata.primarykey import PrimaryKey
from aceql.metadata.remote_database_metadata import RemoteDatabaseMetaData
from aceql.metadata.table import Table
from aceql.metadata.column import Column

__all__ = ["Connection", "Cursor", "Error", "ConnectionOptions", "ProgressIndicator", "ProxyAuth", "SqlNullType",
           "Column", "ExportedKey",
           "ForeignKey", "ImportedKey", "Index", "JdbcDatabaseMetaData", "PrimaryKey", "RemoteDatabaseMetaData",
           "Table"]


def connect(*, url: str, username: str = None, password: str = None, database: str = None, connection_options: ConnectionOptions = None):
    """
    Creates a database connection to the remote AceQL HTTP server.

    Parameters
    ----------
    url : str
        The URL of the AceQL server. Example: https://www.acme.com:9443/aceql.
        The URL may includes all parameters:
        https://www.acme.com:9443/acel?username=my_name&password=my_passwd&database=my_db
    username : str
        The authentication username.
    password : str
        the authentication password.
    database : str
        The remote database name.
    connection_options : ConnectionOptions
            the  supplemental Connection Options (Container that allows define some options: proxies,
            timeout, request headers, etc.)

    Returns
    -------
    Connection
        A connection to the remote database.

    """

    the_connection = Connection(url=url, username=username, password=password, database=database,
                                connection_options=connection_options)
    return the_connection



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2021,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##


class SqlNullType(object):
    """Defines nullable SQL Types when setting a SQL parameter to NULL.

        To set a prepared statement to NULL, set the value to a tuple (None, SqlNullType.<SQL_TYPE>).
        Example: set to NULL an INTEGER column ==>  (None, SqlNullType.INTEGER) """

    BIT = -7
    TINYINT = -6
    SMALLINT = 5
    INTEGER = 4
    BIGINT = -5
    NUMERIC = 2
    REAL = 7
    CHAR = 1
    VARCHAR = 12
    DATE = 91
    TIME = 92
    TIMESTAMP = 93
    BOOLEAN = 16

    BLOB = 2004
    CLOB = 2005
    # FLOAT = 6
    # DOUBLE = 8
    # DECIMAL = 3



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2021,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##
from aceql._private.aceql_http_api import AceQLHttpApi
from aceql.cursor import Cursor
from aceql.connection_options import ConnectionOptions
from aceql.progress_indicator import ProgressIndicator
from aceql.login_url_decoder import LoginUrlDecoder

class Connection(object):
    """Allows to create a database connection to a remote server."""

    def __init__(self, *, url: str, username: str= None, password: str = None, database: str = None,
                 connection_options: ConnectionOptions = None):
        """
        Creates a database connection to the remote AceQL HTTP server.

        Parameters
        ----------
        url : str
            The URL of the AceQL server. Example: https://www.acme.com:9443/aceql.
            The URL may includes all parameters:
            https://www.acme.com:9443/acel?username=my_name&password=my_passwd&database=my_db
        username : str
            The authentication username.
        password : str
            the authentication password.
        database : str
            The remote database name.
            the  supplemental Connection Options (Container that allows define some options: proxies,
            timeout, request headers, etc.)

        Returns
        -------
        Connection
            A connection to the remote database.
        """

        if url is None:
            raise TypeError("url is null!")

        url_connection = LoginUrlDecoder(url)
        if "?" in url:
            url = url_connection.server_url
            if url_connection.username is not None:
                username = url_connection.username
            if url_connection.password is not None:
                password = url_connection.password
            if url_connection.database is not None:
                database = url_connection.database

        if username is None:
            raise TypeError("username is null!")
        if password is None:
            if connection_options is None:
                raise TypeError("password and connection_options cannot be both null!")
            if connection_options.session_id is None:
                raise TypeError("password and connection_options.session_id cannot be both null!")
        if database is None:
            raise TypeError("database is null!")

        self.__url = url
        self.__username = username
        self.__database = database

        self.__aceQLHttpApi = AceQLHttpApi(url=url, username=username, password=password, database=database,
                                           connection_options=connection_options)
        self.__connection_options = connection_options

    @property
    def _get_aceql_http_api(self) -> AceQLHttpApi:
        return self.__aceQLHttpApi

    def get_url(self) -> str :
        """Gets the Serverl URL for this Connection."""
        return self.__url

    def get_username(self) -> str :
        """Gets the username for this Connection."""
        return self.__username

    def get_database(self) -> str :
        """Gets the database for this Connection."""
        return self.__database

    def get_connections_options(self) -> ConnectionOptions:
        """Gets the Connections options."""
        return self.__connection_options

    def cursor(self) -> Cursor:
        """Instantiates and returns a cursor."""
        cursor = Cursor(self, self.__aceQLHttpApi)
        return cursor

    def set_progress_indicator(self, progress_indicator: ProgressIndicator):
        """Allows to set a progress indicator."""
        self.__aceQLHttpApi.set_progress_indicator(progress_indicator)

    def get_progress_indicator(self) -> ProgressIndicator:
        """Returns the progress indicator in use."""
        return self.__aceQLHttpApi.get_progress_indicator()

    def set_auto_commit(self, auto_commit: bool):
        """Sets this connection's auto-commit mode to the given state."""
        self.__aceQLHttpApi.set_auto_commit(auto_commit)

    def get_auto_commit(self):
        """Retrieves the current auto-commit mode."""
        return self.__aceQLHttpApi.get_auto_commit()

    def commit(self):
        """Commit current transaction."""
        self.__aceQLHttpApi.commit()

    def rollback(self):
        """Rollback current transaction."""
        self.__aceQLHttpApi.rollback()

    def _trace(self):
        """Print empty line on trace."""
        self.__aceQLHttpApi.trace("")

    def trace(self, s: str):
        """Prints the string on trace."""
        self.__aceQLHttpApi.trace(s)

    @staticmethod
    def _is_trace_on() -> bool:
        """Says if trace is on."""
        return AceQLHttpApi.is_trace_on()

    @staticmethod
    def _set_trace_on(trace_on: bool):
        """Sets the trace on/off."""
        AceQLHttpApi.set_trace_on(trace_on)

    def get_server_version(self) -> str:
        """Gets the server version of AceQL HTTP."""
        return self.__aceQLHttpApi.get_server_version()

    def close(self):
        """Closes the connection to the remote database but keeps the HTTP session."""
        self.__aceQLHttpApi.close()

    def logout(self):
        """Closes all the connection to the remote database and closes the HTTP session."""
        self.__aceQLHttpApi.logout()

    def get_transaction_isolation(self) -> str:
        """Returns the current transaction isolation level."

           Will be one of the following constants:
                transaction_read_uncommitted,
                transaction_read_committed,
                transaction_repeatable_read,
                transaction_serializable, or
                transaction_none.
        """
        return self.__aceQLHttpApi.get_transaction_isolation()

    def set_transaction_isolation(self, level: str):
        """Sets the transaction isolation level."""
        self.__aceQLHttpApi.set_transaction_isolation(level)

    def get_holdability(self) -> str:
        """Returns the holdability.
         One of hold_cursors_over_commit or close_cursors_at_commit.
        """
        return self.__aceQLHttpApi.get_holdability()

    def set_holdability(self, holdability: str):
        """Sets the holdability."""
        self.__aceQLHttpApi.set_holdability(holdability)

    def is_read_only(self) -> bool:
        """Says if Connection is read-only."""
        return self.__aceQLHttpApi.is_read_only()

    def set_read_only(self, read_only: bool):
        """Allows to put Connection read-only mode."""
        self.__aceQLHttpApi.set_read_only(read_only)

    @staticmethod
    def get_client_version() -> str:
        """Gets the AceQL Cient SDK version."""
        return AceQLHttpApi.get_client_version()

    @staticmethod
    def get_client_version_full() -> str:
        """Gets the AceQL Cient SDK version with the Python version."""
        return AceQLHttpApi.get_client_version_full()



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2021,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##

from dataclasses import dataclass
from typing import Optional


@dataclass
class Index:

    """A SQL Index with it's defining elements."""
    catalog: Optional[str]
    schema: Optional[str]
    indexName: Optional[str]
    tableName: Optional[str]
    nonUnique: Optional[bool]
    indexQualifier: Optional[str]
    type: Optional[str]
    ordinalPosition: Optional[int]
    columnName: Optional[str]
    ascendingOrDescending: Optional[str]
    cardinality: Optional[float]
    pages: Optional[float]
    filterCondition: Optional[str]

    class Meta:

        """Meta class is required."""
        ordered = True

    def __str__(self):
        """The string representation."""
        return "Index [indexName=" + str(self.indexName) + ", tableName=" + str(self.tableName) + ", nonUnique=" + str(
            self.nonUnique) + ", indexQualifier=" + str(self.indexQualifier) + ", type=" + str(
            self.type) + ", ordinalPosition=" + str(self.ordinalPosition) + ", columnName=" + str(
            self.columnName) + ", ascendingOrDescending=" + str(self.ascendingOrDescending) + ", cardinality=" + str(
            self.cardinality) + ", pages=" + str(self.pages) + ", filterCondition=" + str(self.filterCondition) + "]"



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2021,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##

from dataclasses import dataclass
from typing import Optional, List
from aceql.metadata.column import Column
from aceql.metadata.exportedkey import ExportedKey
from aceql.metadata.importedkey import ImportedKey
from aceql.metadata.index import Index
from aceql.metadata.primarykey import PrimaryKey


@dataclass
class Table:

    """A SQL Table with it's defining elements."""
    TABLE = "TABLE";
    VIEW = "VIEW";
    tableName: Optional[str]
    tableType: Optional[str]
    remarks: Optional[str]
    columns: Optional[List[Column]]
    primaryKeys: Optional[List[PrimaryKey]]
    indexes: Optional[List[Index]]
    importedforeignKeys: Optional[List[ImportedKey]]
    exportedforeignKeys: Optional[List[ExportedKey]]
    catalog: Optional[str]
    schema: Optional[str]

    class Meta:

        """Meta class is required."""
        ordered = True

    def __str__(self):
        """The string representation."""
        return "Table [tableName=" + str(self.tableName) + ", tableType=" + str(self.tableType) + ", remarks=" + str(
            self.remarks) + ", columns=" + str(self.columns) + ", primaryKeys=" + str(
            self.primaryKeys) + ", indexes=" + str(self.indexes) + ", importedforeignKeys=" + str(
            self.importedforeignKeys) + ", exportedforeignKeys=" + str(self.exportedforeignKeys) + ", catalog=" + str(
            self.catalog) + ", schema=" + str(self.schema) + "]"



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2021,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##

import uuid
from os import sep
import os
import gzip
import sys
from io import open


class FileUtil(object):
    """Misc file utilities."""

    UNZIP_MODE_PY_2 = "rb"
    WRITE_MODE_PY_2 = "wb"

    UNZIP_MODE_PY_3 = "rt"
    WRITE_MODE_PY_3 = "wt"

    @staticmethod
    def get_unique_id() -> str:
        x = uuid.uuid4()
        return str(x)

    @staticmethod
    def get_user_home_dot_kawansoft_dir() -> str:
        # user.home
        # home = str(Path.home())
        home = os.path.expanduser("~")
        # print("home2: " + home2)

        # File.separator
        home_kawan_soft = home + sep + ".kawansoft"

        if not os.path.isdir(home_kawan_soft):
            os.mkdir(home_kawan_soft)

        return home_kawan_soft

    @staticmethod
    def get_kawansoft_temp_dir() -> str:
        home_kawan_soft = FileUtil.get_user_home_dot_kawansoft_dir()

        # File.separator
        home_kawan_soft_tmp = home_kawan_soft + sep + "tmp"

        if not os.path.isdir(home_kawan_soft_tmp):
            os.mkdir(home_kawan_soft_tmp)

        return home_kawan_soft_tmp

    @staticmethod
    def build_result_set_file() -> str:
        the_file = FileUtil.get_kawansoft_temp_dir() + sep + "pc-result-set-" + FileUtil.get_unique_id() + ".txt"
        return the_file

    @staticmethod
    def decompress(file_in: str, file_out: str):
        """Decompress GZIP text file into a text file."""

        if file_in is None:
            raise TypeError("file_in is null!")

        if not os.path.isfile(file_in):
            raise IOError("Compressed file_in does not exist: " + str(file_in))

        # P2 = rb / wb
        # P3 = rt / wt
        gzip_mode = FileUtil.get_unzip_mode()
        write_mode = FileUtil.get_write_mode()

        with gzip.open(file_in, gzip_mode) as f:
            with open(file_out, write_mode) as out:
                while True:
                    line = f.readline()
                    if line == '':
                        break
                        # print(line)
                    out.write(line)

    @staticmethod
    def get_unzip_mode() -> str:
        if FileUtil.is_python_3():
            return FileUtil.UNZIP_MODE_PY_3
        else:
            return FileUtil.UNZIP_MODE_PY_2

    @staticmethod
    def get_write_mode()-> str:
        if FileUtil.is_python_3():
            return FileUtil.WRITE_MODE_PY_3
        else:
            return FileUtil.WRITE_MODE_PY_2

    @staticmethod
    def is_python_2() -> bool:
        if sys.version_info[0] < 3:
            return True

    @staticmethod
    def is_python_3() -> bool:
        if sys.version_info[0] >= 3:
            return True



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2021,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##

from dataclasses import dataclass
from typing import Optional


@dataclass
class JdbcDatabaseMetaData:

    """
    Contains main SQL meta info sent by remote JDBC Driver. The info matches the DatabaseMetaData Java class [1] main values.
    [1]: https://docs.oracle.com/javase/8/docs/api/java/sql/DatabaseMetaData.html
    """
    getURL: Optional[str]
    isReadOnly: Optional[bool]
    allProceduresAreCallable: Optional[bool]
    allTablesAreSelectable: Optional[bool]
    getUserName: Optional[str]
    nullsAreSortedHigh: Optional[bool]
    nullsAreSortedLow: Optional[bool]
    nullsAreSortedAtStart: Optional[bool]
    nullsAreSortedAtEnd: Optional[bool]
    getDatabaseProductName: Optional[str]
    getDatabaseProductVersion: Optional[str]
    getDriverName: Optional[str]
    getDriverVersion: Optional[str]
    getDriverMajorVersion: Optional[int]
    getDriverMinorVersion: Optional[int]
    usesLocalFiles: Optional[bool]
    usesLocalFilePerTable: Optional[bool]
    supportsMixedCaseIdentifiers: Optional[bool]
    storesUpperCaseIdentifiers: Optional[bool]
    storesLowerCaseIdentifiers: Optional[bool]
    storesMixedCaseIdentifiers: Optional[bool]
    supportsMixedCaseQuotedIdentifiers: Optional[bool]
    storesUpperCaseQuotedIdentifiers: Optional[bool]
    storesLowerCaseQuotedIdentifiers: Optional[bool]
    storesMixedCaseQuotedIdentifiers: Optional[bool]
    getIdentifierQuoteString: Optional[str]
    getSQLKeywords: Optional[str]
    getNumericFunctions: Optional[str]
    getStringFunctions: Optional[str]
    getSystemFunctions: Optional[str]
    getTimeDateFunctions: Optional[str]
    getSearchStringEscape: Optional[str]
    getExtraNameCharacters: Optional[str]
    supportsAlterTableWithAddColumn: Optional[bool]
    supportsAlterTableWithDropColumn: Optional[bool]
    supportsColumnAliasing: Optional[bool]
    nullPlusNonNullIsNull: Optional[bool]
    supportsConvert: Optional[bool]
    supportsTableCorrelationNames: Optional[bool]
    supportsDifferentTableCorrelationNames: Optional[bool]
    supportsExpressionsInOrderBy: Optional[bool]
    supportsOrderByUnrelated: Optional[bool]
    supportsGroupBy: Optional[bool]
    supportsGroupByUnrelated: Optional[bool]
    supportsGroupByBeyondSelect: Optional[bool]
    supportsLikeEscapeClause: Optional[bool]
    supportsMultipleResultSets: Optional[bool]
    supportsMultipleTransactions: Optional[bool]
    supportsNonNullableColumns: Optional[bool]
    supportsMinimumSQLGrammar: Optional[bool]
    supportsCoreSQLGrammar: Optional[bool]
    supportsExtendedSQLGrammar: Optional[bool]
    supportsANSI92EntryLevelSQL: Optional[bool]
    supportsANSI92IntermediateSQL: Optional[bool]
    supportsANSI92FullSQL: Optional[bool]
    supportsIntegrityEnhancementFacility: Optional[bool]
    supportsOuterJoins: Optional[bool]
    supportsFullOuterJoins: Optional[bool]
    supportsLimitedOuterJoins: Optional[bool]
    getSchemaTerm: Optional[str]
    getProcedureTerm: Optional[str]
    getCatalogTerm: Optional[str]
    isCatalogAtStart: Optional[bool]
    getCatalogSeparator: Optional[str]
    supportsSchemasInDataManipulation: Optional[bool]
    supportsSchemasInProcedureCalls: Optional[bool]
    supportsSchemasInTableDefinitions: Optional[bool]
    supportsSchemasInIndexDefinitions: Optional[bool]
    supportsSchemasInPrivilegeDefinitions: Optional[bool]
    supportsCatalogsInDataManipulation: Optional[bool]
    supportsCatalogsInProcedureCalls: Optional[bool]
    supportsCatalogsInTableDefinitions: Optional[bool]
    supportsCatalogsInIndexDefinitions: Optional[bool]
    supportsCatalogsInPrivilegeDefinitions: Optional[bool]
    supportsPositionedDelete: Optional[bool]
    supportsPositionedUpdate: Optional[bool]
    supportsSelectForUpdate: Optional[bool]
    supportsStoredProcedures: Optional[bool]
    supportsSubqueriesInComparisons: Optional[bool]
    supportsSubqueriesInExists: Optional[bool]
    supportsSubqueriesInIns: Optional[bool]
    supportsSubqueriesInQuantifieds: Optional[bool]
    supportsCorrelatedSubqueries: Optional[bool]
    supportsUnion: Optional[bool]
    supportsUnionAll: Optional[bool]
    supportsOpenCursorsAcrossCommit: Optional[bool]
    supportsOpenCursorsAcrossRollback: Optional[bool]
    supportsOpenStatementsAcrossCommit: Optional[bool]
    supportsOpenStatementsAcrossRollback: Optional[bool]
    getMaxBinaryLiteralLength: Optional[int]
    getMaxCharLiteralLength: Optional[int]
    getMaxColumnNameLength: Optional[int]
    getMaxColumnsInGroupBy: Optional[int]
    getMaxColumnsInIndex: Optional[int]
    getMaxColumnsInOrderBy: Optional[int]
    getMaxColumnsInSelect: Optional[int]
    getMaxColumnsInTable: Optional[int]
    getMaxConnections: Optional[int]
    getMaxCursorNameLength: Optional[int]
    getMaxIndexLength: Optional[int]
    getMaxSchemaNameLength: Optional[int]
    getMaxProcedureNameLength: Optional[int]
    getMaxCatalogNameLength: Optional[int]
    getMaxRowSize: Optional[int]
    doesMaxRowSizeIncludeBlobs: Optional[bool]
    getMaxStatementLength: Optional[int]
    getMaxStatements: Optional[int]
    getMaxTableNameLength: Optional[int]
    getMaxTablesInSelect: Optional[int]
    getMaxUserNameLength: Optional[int]
    getDefaultTransactionIsolation: Optional[int]
    supportsTransactions: Optional[bool]
    supportsDataDefinitionAndDataManipulationTransactions: Optional[bool]
    supportsDataManipulationTransactionsOnly: Optional[bool]
    dataDefinitionCausesTransactionCommit: Optional[bool]
    dataDefinitionIgnoredInTransactions: Optional[bool]
    supportsBatchUpdates: Optional[bool]
    supportsSavepoints: Optional[bool]
    supportsNamedParameters: Optional[bool]
    supportsMultipleOpenResults: Optional[bool]
    supportsGetGeneratedKeys: Optional[bool]
    getDatabaseMajorVersion: Optional[int]
    getDatabaseMinorVersion: Optional[int]
    getJDBCMajorVersion: Optional[int]
    getJDBCMinorVersion: Optional[int]
    getSQLStateType: Optional[int]
    locatorsUpdateCopy: Optional[bool]
    supportsStatementPooling: Optional[bool]
    supportsStoredFunctionsUsingCallSyntax: Optional[bool]
    autoCommitFailureClosesAllResultSets: Optional[bool]
    getResultSetHoldability: Optional[int]

    class Meta:
        ordered = True

    def __str__(self):
        """ The string representation."""
        return "JdbcDatabaseMetaData [getURL=" + str(self.getURL) + ", isReadOnly=" + str(
            self.isReadOnly) + ", allProceduresAreCallable=" + str(
            self.allProceduresAreCallable) + ", allTablesAreSelectable=" + str(
            self.allTablesAreSelectable) + ", getUserName=" + str(self.getUserName) + ", nullsAreSortedHigh=" + str(
            self.nullsAreSortedHigh) + ", nullsAreSortedLow=" + str(
            self.nullsAreSortedLow) + ", nullsAreSortedAtStart=" + str(
            self.nullsAreSortedAtStart) + ", nullsAreSortedAtEnd=" + str(
            self.nullsAreSortedAtEnd) + ", getDatabaseProductName=" + str(
            self.getDatabaseProductName) + ", getDatabaseProductVersion=" + str(
            self.getDatabaseProductVersion) + ", getDriverName=" + str(
            self.getDriverName) + ", getDriverVersion=" + str(self.getDriverVersion) + ", getDriverMajorVersion=" + str(
            self.getDriverMajorVersion) + ", getDriverMinorVersion=" + str(
            self.getDriverMinorVersion) + ", usesLocalFiles=" + str(
            self.usesLocalFiles) + ", usesLocalFilePerTable=" + str(
            self.usesLocalFilePerTable) + ", supportsMixedCaseIdentifiers=" + str(
            self.supportsMixedCaseIdentifiers) + ", storesUpperCaseIdentifiers=" + str(
            self.storesUpperCaseIdentifiers) + ", storesLowerCaseIdentifiers=" + str(
            self.storesLowerCaseIdentifiers) + ", storesMixedCaseIdentifiers=" + str(
            self.storesMixedCaseIdentifiers) + ", supportsMixedCaseQuotedIdentifiers=" + str(
            self.supportsMixedCaseQuotedIdentifiers) + ", storesUpperCaseQuotedIdentifiers=" + str(
            self.storesUpperCaseQuotedIdentifiers) + ", storesLowerCaseQuotedIdentifiers=" + str(
            self.storesLowerCaseQuotedIdentifiers) + ", storesMixedCaseQuotedIdentifiers=" + str(
            self.storesMixedCaseQuotedIdentifiers) + ", getIdentifierQuoteString=" + str(
            self.getIdentifierQuoteString) + ", getSQLKeywords=" + str(
            self.getSQLKeywords) + ", getNumericFunctions=" + str(
            self.getNumericFunctions) + ", getStringFunctions=" + str(
            self.getStringFunctions) + ", getSystemFunctions=" + str(
            self.getSystemFunctions) + ", getTimeDateFunctions=" + str(
            self.getTimeDateFunctions) + ", getSearchStringEscape=" + str(
            self.getSearchStringEscape) + ", getExtraNameCharacters=" + str(
            self.getExtraNameCharacters) + ", supportsAlterTableWithAddColumn=" + str(
            self.supportsAlterTableWithAddColumn) + ", supportsAlterTableWithDropColumn=" + str(
            self.supportsAlterTableWithDropColumn) + ", supportsColumnAliasing=" + str(
            self.supportsColumnAliasing) + ", nullPlusNonNullIsNull=" + str(
            self.nullPlusNonNullIsNull) + ", supportsConvert=" + str(
            self.supportsConvert) + ", supportsTableCorrelationNames=" + str(
            self.supportsTableCorrelationNames) + ", supportsDifferentTableCorrelationNames=" + str(
            self.supportsDifferentTableCorrelationNames) + ", supportsExpressionsInOrderBy=" + str(
            self.supportsExpressionsInOrderBy) + ", supportsOrderByUnrelated=" + str(
            self.supportsOrderByUnrelated) + ", supportsGroupBy=" + str(
            self.supportsGroupBy) + ", supportsGroupByUnrelated=" + str(
            self.supportsGroupByUnrelated) + ", supportsGroupByBeyondSelect=" + str(
            self.supportsGroupByBeyondSelect) + ", supportsLikeEscapeClause=" + str(
            self.supportsLikeEscapeClause) + ", supportsMultipleResultSets=" + str(
            self.supportsMultipleResultSets) + ", supportsMultipleTransactions=" + str(
            self.supportsMultipleTransactions) + ", supportsNonNullableColumns=" + str(
            self.supportsNonNullableColumns) + ", supportsMinimumSQLGrammar=" + str(
            self.supportsMinimumSQLGrammar) + ", supportsCoreSQLGrammar=" + str(
            self.supportsCoreSQLGrammar) + ", supportsExtendedSQLGrammar=" + str(
            self.supportsExtendedSQLGrammar) + ", supportsANSI92EntryLevelSQL=" + str(
            self.supportsANSI92EntryLevelSQL) + ", supportsANSI92IntermediateSQL=" + str(
            self.supportsANSI92IntermediateSQL) + ", supportsANSI92FullSQL=" + str(
            self.supportsANSI92FullSQL) + ", supportsIntegrityEnhancementFacility=" + str(
            self.supportsIntegrityEnhancementFacility) + ", supportsOuterJoins=" + str(
            self.supportsOuterJoins) + ", supportsFullOuterJoins=" + str(
            self.supportsFullOuterJoins) + ", supportsLimitedOuterJoins=" + str(
            self.supportsLimitedOuterJoins) + ", getSchemaTerm=" + str(
            self.getSchemaTerm) + ", getProcedureTerm=" + str(self.getProcedureTerm) + ", getCatalogTerm=" + str(
            self.getCatalogTerm) + ", isCatalogAtStart=" + str(self.isCatalogAtStart) + ", getCatalogSeparator=" + str(
            self.getCatalogSeparator) + ", supportsSchemasInDataManipulation=" + str(
            self.supportsSchemasInDataManipulation) + ", supportsSchemasInProcedureCalls=" + str(
            self.supportsSchemasInProcedureCalls) + ", supportsSchemasInTableDefinitions=" + str(
            self.supportsSchemasInTableDefinitions) + ", supportsSchemasInIndexDefinitions=" + str(
            self.supportsSchemasInIndexDefinitions) + ", supportsSchemasInPrivilegeDefinitions=" + str(
            self.supportsSchemasInPrivilegeDefinitions) + ", supportsCatalogsInDataManipulation=" + str(
            self.supportsCatalogsInDataManipulation) + ", supportsCatalogsInProcedureCalls=" + str(
            self.supportsCatalogsInProcedureCalls) + ", supportsCatalogsInTableDefinitions=" + str(
            self.supportsCatalogsInTableDefinitions) + ", supportsCatalogsInIndexDefinitions=" + str(
            self.supportsCatalogsInIndexDefinitions) + ", supportsCatalogsInPrivilegeDefinitions=" + str(
            self.supportsCatalogsInPrivilegeDefinitions) + ", supportsPositionedDelete=" + str(
            self.supportsPositionedDelete) + ", supportsPositionedUpdate=" + str(
            self.supportsPositionedUpdate) + ", supportsSelectForUpdate=" + str(
            self.supportsSelectForUpdate) + ", supportsStoredProcedures=" + str(
            self.supportsStoredProcedures) + ", supportsSubqueriesInComparisons=" + str(
            self.supportsSubqueriesInComparisons) + ", supportsSubqueriesInExists=" + str(
            self.supportsSubqueriesInExists) + ", supportsSubqueriesInIns=" + str(
            self.supportsSubqueriesInIns) + ", supportsSubqueriesInQuantifieds=" + str(
            self.supportsSubqueriesInQuantifieds) + ", supportsCorrelatedSubqueries=" + str(
            self.supportsCorrelatedSubqueries) + ", supportsUnion=" + str(
            self.supportsUnion) + ", supportsUnionAll=" + str(
            self.supportsUnionAll) + ", supportsOpenCursorsAcrossCommit=" + str(
            self.supportsOpenCursorsAcrossCommit) + ", supportsOpenCursorsAcrossRollback=" + str(
            self.supportsOpenCursorsAcrossRollback) + ", supportsOpenStatementsAcrossCommit=" + str(
            self.supportsOpenStatementsAcrossCommit) + ", supportsOpenStatementsAcrossRollback=" + str(
            self.supportsOpenStatementsAcrossRollback) + ", getMaxBinaryLiteralLength=" + str(
            self.getMaxBinaryLiteralLength) + ", getMaxCharLiteralLength=" + str(
            self.getMaxCharLiteralLength) + ", getMaxColumnNameLength=" + str(
            self.getMaxColumnNameLength) + ", getMaxColumnsInGroupBy=" + str(
            self.getMaxColumnsInGroupBy) + ", getMaxColumnsInIndex=" + str(
            self.getMaxColumnsInIndex) + ", getMaxColumnsInOrderBy=" + str(
            self.getMaxColumnsInOrderBy) + ", getMaxColumnsInSelect=" + str(
            self.getMaxColumnsInSelect) + ", getMaxColumnsInTable=" + str(
            self.getMaxColumnsInTable) + ", getMaxConnections=" + str(
            self.getMaxConnections) + ", getMaxCursorNameLength=" + str(
            self.getMaxCursorNameLength) + ", getMaxIndexLength=" + str(
            self.getMaxIndexLength) + ", getMaxSchemaNameLength=" + str(
            self.getMaxSchemaNameLength) + ", getMaxProcedureNameLength=" + str(
            self.getMaxProcedureNameLength) + ", getMaxCatalogNameLength=" + str(
            self.getMaxCatalogNameLength) + ", getMaxRowSize=" + str(
            self.getMaxRowSize) + ", doesMaxRowSizeIncludeBlobs=" + str(
            self.doesMaxRowSizeIncludeBlobs) + ", getMaxStatementLength=" + str(
            self.getMaxStatementLength) + ", getMaxStatements=" + str(
            self.getMaxStatements) + ", getMaxTableNameLength=" + str(
            self.getMaxTableNameLength) + ", getMaxTablesInSelect=" + str(
            self.getMaxTablesInSelect) + ", getMaxUserNameLength=" + str(
            self.getMaxUserNameLength) + ", getDefaultTransactionIsolation=" + str(
            self.getDefaultTransactionIsolation) + ", supportsTransactions=" + str(
            self.supportsTransactions) + ", supportsDataDefinitionAndDataManipulationTransactions=" + str(
            self.supportsDataDefinitionAndDataManipulationTransactions) + ", supportsDataManipulationTransactionsOnly=" + str(
            self.supportsDataManipulationTransactionsOnly) + ", dataDefinitionCausesTransactionCommit=" + str(
            self.dataDefinitionCausesTransactionCommit) + ", dataDefinitionIgnoredInTransactions=" + str(
            self.dataDefinitionIgnoredInTransactions) + ", supportsBatchUpdates=" + str(
            self.supportsBatchUpdates) + ", supportsSavepoints=" + str(
            self.supportsSavepoints) + ", supportsNamedParameters=" + str(
            self.supportsNamedParameters) + ", supportsMultipleOpenResults=" + str(
            self.supportsMultipleOpenResults) + ", supportsGetGeneratedKeys=" + str(
            self.supportsGetGeneratedKeys) + ", getDatabaseMajorVersion=" + str(
            self.getDatabaseMajorVersion) + ", getDatabaseMinorVersion=" + str(
            self.getDatabaseMinorVersion) + ", getJDBCMajorVersion=" + str(
            self.getJDBCMajorVersion) + ", getJDBCMinorVersion=" + str(
            self.getJDBCMinorVersion) + ", getSQLStateType=" + str(
            self.getSQLStateType) + ", locatorsUpdateCopy=" + str(
            self.locatorsUpdateCopy) + ", supportsStatementPooling=" + str(
            self.supportsStatementPooling) + ", supportsStoredFunctionsUsingCallSyntax=" + str(
            self.supportsStoredFunctionsUsingCallSyntax) + ", autoCommitFailureClosesAllResultSets=" + str(
            self.autoCommitFailureClosesAllResultSets) + ", getResultSetHoldability=" + str(
            self.getResultSetHoldability) + "]"



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2021,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##

from urllib.parse import urlsplit, parse_qsl


class LoginUrlDecoder(object):
    """Allows to split an URL into the basic login info: login URL, username, password and database."""

    def __init__(self, url: str):
        if url is None:
            raise TypeError("url is null!")

        my_dict: dict = dict(parse_qsl(urlsplit(url).query))

        self.__server_url: str = url.split('?')[0]

        self.__username = None
        self.__password = None
        self.__database = None

        for theKey, theValue in my_dict.items():
            if theKey == "username":
                self.__username = theValue
            if theKey == "password":
                self.__password = theValue
            if theKey == "database":
                self.__database = theValue

    @property
    def server_url(self) -> str:
        """Returns the server login url"""
        return self.__server_url

    @property
    def username(self) -> str:
        """Returns the username"""
        return self.__username

    @property
    def password(self) -> str:
        """Returns the password"""
        return self.__password

    @property
    def database(self) -> str:
        """Returns the database name"""
        return self.__database




#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2021,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##
from aceql.proxy_auth import ProxyAuth


class ConnectionOptions(object):
    """Allows to create the Connection Options for the current session."""

    def __init__(self, *, proxies: dict = None, auth: ProxyAuth = None, session_id: str = None,
                 gzip_result: bool = False, timeout=None, request_headers: dict = None):
        """
            Creates the Connection Options for the current session

            Parameters
            ----------
            proxies : dict
                The proxy to use, can  be an authenticated proxy.
            auth : ProxyAuth
                aceql.ProxyAuth instance with proxy (username, password).
            session_id : str
                A session_id generated by AceQL login method on server. To be used for authentication without password.
            gzip_result : bool
                Says if Remote ResultSet will be zipped before returned to client side.
            timeout
                the HTTP connection timeout as defined in Requests library. None means no timeout is used
                (default value).
                See https://docs.python-requests.org/en/master/user/advanced/#timeouts.
            request_headers : dict
                the request headers to send to server for validation.

            Returns
            -------
            ConnectionOptions
                An object containing all connections options for this session
        """

        self.__proxies = proxies
        self.__auth = auth
        self.__session_id = session_id
        self.__gzip_result = gzip_result
        self.__timeout = timeout
        self.__request_headers = request_headers

    @property
    def proxies(self) -> str:
        return self.__proxies

    @property
    def auth(self) -> str:
        return self.__auth

    @property
    def session_id(self) -> str:
        return self.__session_id

    @property
    def gzip_result(self) -> bool:
        return self.__gzip_result

    @property
    def timeout(self):
        return self.__timeout

    @property
    def request_headers(self) -> dict:
        return self.__request_headers

    def __str__(self):
        """ The string representation."""
        return "ConnectionOptions [proxies=" + str(self.proxies) + ", auth=" + str(self.auth) \
               + ", session_id=" + str(self.session_id) \
               + ", gzip_result=" + str(self.gzip_result) + ", timeout=" + str(self.timeout) \
               + ", request_headers=" + str(self.request_headers) + "]"



# -*- coding: utf-8 -*-
#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2021,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##

import json
import os
from io import open

# -*- coding: utf-8 -*-
from aceql._private.aceql_debug import AceQLDebug


class StreamResultAnalyzer(object):
    """
     Class <see cref="StreamResultAnalyzer"/>. Allows to analyze the result of a downloaded result of a SQL query stored in a local PC file.
    """
    #
    # The error identifier
    # The error message
    # The stack trace
    #
    # The JSON file containing Result Set
    def __init__(self, filename: str, http_status_code: int):
        """
         Initializes a new instance of the <see cref="StreamResultAnalyzer"/> class.

         <param name="filename">The file to analyze.</param>
         <param name="httpStatusCode">The http status code.</param>
         <exception cref="System.ArgumentNullException">The file is null.</exception>
        """

        if filename is None:
            raise TypeError("filename is null!")

        if not os.path.isfile(filename):
            raise IOError("filename does not exist: " + str(filename))

        self.__filename = filename
        self.__http_status_code = http_status_code

        self.__error_type = None
        self.__error_message = None
        self.__stack_trace = None

    def is_status_ok(self) -> bool:
        """
         Determines whether the SQL correctly executed on server side.
         <returns><c>true</c> if [is status ok]; otherwise, <c>false</c>.</returns>
        """
        with open(self.__filename, mode="r", encoding="utf-8") as fd:
            status_ok = False
            while True:
                s = fd.readline()
                if s == '':
                    break
                s = s.strip()
                AceQLDebug.debug(s)
                if s.startswith("\"status\":"):
                    AceQLDebug.debug("status: " + s + "!")
                    if s.endswith("\"OK\","):
                        status_ok = True
                        break

        if not status_ok:
            self.parse_error_keywords()

        return status_ok

    def parse_error_keywords(self):
        """
         Parses the error keywords.
         <param name="reader">The reader.</param>
        """
        with open(self.__filename, mode="r", encoding="utf-8") as fd:
            s = fd.read()
            j = json.loads(s)
            self.__error_type = j["error_type"]
            self.__error_message = j["error_message"]

            try:
                self.__stack_trace = j["stack_trace"]
            except Exception as e:
                print("Exception when accessing j[stack_trace]:")
                print(str(e))

            return

    def get_error_message(self) -> str:
        """
         Gets the error message.
         <returns>The error message</returns>
        """
        return self.__error_message

    def get_error_type(self) -> int:
        """
         Gets the error type.
         <returns>The error type.</returns>
        """
        return self.__error_type

    def get_stack_trace(self) -> str:
        """
         Gets the remote stack trace.
         <returns>The remote stack trace.</returns>
        """
        return self.__stack_trace



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2021,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##

from dataclasses import dataclass
from typing import Optional


@dataclass
class ForeignKey:
    """A SQL Foreign Key."""
    catalog: Optional[str]
    schema: Optional[str]
    primaryKeyTable: Optional[str]
    primaryKeyColumn: Optional[str]
    foreignKeyCatalog: Optional[str]
    foreignKeySchema: Optional[str]
    foreignKeyTable: Optional[str]
    foreignKeyColumn: Optional[str]
    keySequence: Optional[int]
    updateRule: Optional[str]
    deleteRule: Optional[str]
    foreignKeyName: Optional[str]
    primaryKeyName: Optional[str]
    deferrability: Optional[int]

    class Meta:
        """Meta class is required."""
        ordered = True

    def __str__(self):
        """The string representation."""
        return "ForeignKey [primaryKeyTable=" + str(self.primaryKeyTable) + ", primaryKeyColumn=" + str(
            self.primaryKeyColumn) + ", foreignKeyCatalog=" + str(self.foreignKeyCatalog) + ", foreignKeySchema=" + str(
            self.foreignKeySchema) + ", foreignKeyTable=" + str(self.foreignKeyTable) + ", foreignKeyColumn=" + str(
            self.foreignKeyColumn) + ", keySequence=" + str(self.keySequence) + ", updateRule=" + str(
            self.updateRule) + ", deleteRule=" + str(self.deleteRule) + ", foreignKeyName=" + str(
            self.foreignKeyName) + ", primaryKeyName=" + str(self.primaryKeyName) + ", deferrability=" + str(
            self.deferrability) + "]"



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2021,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##

from dataclasses import dataclass
from typing import Optional


@dataclass
class Column:

    """A SQL Column with it's defining elements."""
    catalog: Optional[str]
    schema: Optional[str]
    columnName: Optional[str]
    tableName: Optional[str]
    typeName: Optional[str]
    size: Optional[int]
    decimalDigits: Optional[int]
    radix: Optional[int]
    nullable: Optional[str]
    remarks: Optional[str]
    defaultValue: Optional[str]
    charOctetLength: Optional[int]
    ordinalPosition: Optional[int]
    isNullable: Optional[str]
    scopeCatalog: Optional[str]
    scopeSchema: Optional[str]
    scopeTable: Optional[str]
    sourceDataType: Optional[int]
    isAutoincrement: Optional[str]

    class Meta:

        """Meta class is required."""
        ordered = True

    def __str__(self):
        """The string representation."""
        return "Column [columnName=" + str(self.columnName) + ", tableName=" + str(
            self.tableName) + ", typeName=" + str(self.typeName) + ", size=" + str(
            self.size) + ", decimalDigits=" + str(self.decimalDigits) + ", radix=" + str(
            self.radix) + ", nullable=" + str(self.nullable) + ", remarks=" + str(
            self.remarks) + ", defaultValue=" + str(self.defaultValue) + ", charOctetLength=" + str(
            self.charOctetLength) + ", ordinalPosition=" + str(self.ordinalPosition) + ", isNullable=" + str(
            self.isNullable) + ", scopeCatalog=" + str(self.scopeCatalog) + ", scopeSchema=" + str(
            self.scopeSchema) + ", scopeTable=" + str(self.scopeTable) + ", sourceDataType=" + str(
            self.sourceDataType) + ", isAutoincrement=" + str(self.isAutoincrement) + "]"



from setuptools import setup
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='aceql',
    long_description=long_description,
    long_description_content_type='text/markdown',
    version='4.2',
    packages=['aceql', 'aceql._private', 'aceql.metadata'],
    url='https://github.com/kawansoft/aceql-py',
    license='Apache 2.0',

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Database',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
    ],

    keywords='DATABASE SQL HTTP CLIENT-SERVER',
    install_requires=[
        'requests>=2.18.4,<3.0.0',
        'requests_toolbelt>=0.8.0',
        'pytz>=2017.3',
        'marshmallow',
        'ijson'
    ],

    package_data={
        # If any package contains *.txt or *.png or s*.rst files, include them:
        '': ['*.txt', '*.png', '*.md', '*.rst']
    },

    author='KawanSoft',
    author_email='contact@kawansoft.com',
    description='Python 3 Client toolkit for easy access of remote SQL databases managed with AceQL HTTP.'
)



#
# This file is part of AceQL Python Client SDK.
# AceQL Python Client SDK: Remote SQL access over HTTP with AceQL HTTP.
# Copyright (C) 2021,  KawanSoft SAS
# (http://www.kawansoft.com). All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##

from aceql._private.parms import Parms


class AceQLDebug(object):
    """debug class"""

    # stack = inspect.stack()
    # the_class = stack[1][0].f_locals["self"].__class__
    # the_method = stack[1][0].f_code.co_name

    # print("I was called by {}.{}()".format(str(calling_class),
    # calling_code_name))
    ## => I was called by A.a()

    @staticmethod
    def debug_empty():
        if Parms.DEBUG_ON:
            print("debug> ")

    @staticmethod
    def debug(s: str):
        if Parms.DEBUG_ON:
            print("debug> " + str(s))

